/*
Copyright (c) 2007 Matt Williams matt@milliams.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License or (at
your option) version 3 or, at the discretion of KDE e.V. (which shall
act as a proxy as in section 14 of the GPLv3), any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KQTIFLOWSTATIC_H
#define KQTIFLOWSTATIC_H

#include "kqtiflow.h"

#include <kqti_export.h>

#include <QDomElement>

/**
 * \class KQtiFlowStatic
 * A sub-class of flow that excludes interactions (KQtiInteraction)
 * 
 * For now, this class simply contains a QDomElement and provides an interface
 * to it. It doesn't yet exclude interactions or anything.
 * 
 * http://www.imsglobal.org/question/qtiv2p1pd2/imsqti_infov2p1pd2.html#element10124
 */

class KQTI_EXPORT KQtiFlowStatic : public KQtiFlow
{
    public:
        KQtiFlowStatic(const QDomElement& element);
        virtual ~KQtiFlowStatic() {}

        /**
         * \return the QDomElement stored in the class. To display this static text, do
         * KQtiFlowStatic::element().text(). So, if the element was
         * <code><h1>Hello <b>Qt</b> <![CDATA[<xml is cool>]]></h1></code>, it
         * would return <code>Hello Qt &lt;xml is cool&gt;</code>
         */
        QDomElement element() const;
        
    protected:
        QDomElement mElement;
};

#endif
