/*
Copyright (c) 2007 Matt Williams matt@milliams.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License or (at
your option) version 3 or, at the discretion of KDE e.V. (which shall
act as a proxy as in section 14 of the GPLv3), any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "kqtiitembody.h"

#include "kqtichoiceinteraction.h"
#include "kqtiblockstatic.h"

KQtiItemBody::KQtiItemBody()
{
}

KQtiItemBody::KQtiItemBody(const QDomElement& element)
{
    for(uint i=0; i<element.childNodes().length(); i++)
    {
        QDomElement childElement = element.childNodes().item(i).toElement();
        if(childElement.tagName() == "choiceInteraction")
        {
            KQtiChoiceInteraction* choiceInteraction = new KQtiChoiceInteraction(childElement);
            addBlock(choiceInteraction);
        }
        else
        {
            KQtiBlockStatic* blockStatic = new KQtiBlockStatic(childElement);
            addBlock(blockStatic);
        }
    }
}


KQtiItemBody::~KQtiItemBody()
{
    
}

void KQtiItemBody::addBlock(KQtiBlock* block)
{
    mBlocks.append(block);
}

QDomElement KQtiItemBody::element() const
{
    QDomDocument doc;
    
    QDomElement itemBodyElement = doc.createElement("itemBody");
    
    foreach(KQtiBlock* block, mBlocks)
    {
        itemBodyElement.appendChild(block->element());
    }
    
    return itemBodyElement;
}
