/*
Copyright (c) 2007 Matt Williams matt@milliams.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License or (at
your option) version 3 or, at the discretion of KDE e.V. (which shall
act as a proxy as in section 14 of the GPLv3), any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "kqtitestpart.h"

#include "kqtiassessmentsection.h"

KQtiTestPart::KQtiTestPart(const QString& navigationMode, const QString& submissionMode) :
        mNavigationMode(navigationMode),
        mSubmissionMode(submissionMode)/*,
        mIdentifier(identifier)*/
{
}

KQtiTestPart::KQtiTestPart(const QDomElement& element)
{
    mNavigationMode = element.attribute("navigationMode");
    mSubmissionMode = element.attribute("submissionMode");
    mIdentifier = element.attribute("identifier");
    
    for(int i=0; i<element.elementsByTagName("assessmentSection").size(); i++)
    {
        QDomElement assessmentSection = element.elementsByTagName("assessmentSection").item(i).toElement();
        addAssessmentSection(new KQtiAssessmentSection(assessmentSection));
    }
    //mAssessmentSection = new KQtiAssessmentSection(&element->firstChildElement("assessmentSection"));
}


KQtiTestPart::~KQtiTestPart()
{
    //delete mAssessmentSection;
    //TODO delete all mAssessmentSections
}

void KQtiTestPart::addAssessmentSection(KQtiAssessmentSection* assessmentSection)
{
    mAssessmentSections.append(assessmentSection);
}

QDomElement KQtiTestPart::element() const
{
    QDomDocument doc;
    
    QDomElement testPartElement = doc.createElement("testPart");
    testPartElement.setAttribute("navigationMode", mNavigationMode);
    testPartElement.setAttribute("submissionMode", mSubmissionMode);
    testPartElement.setAttribute("identifier", mIdentifier);
    
    foreach(KQtiAssessmentSection* assessmentSection, mAssessmentSections)
    {
        testPartElement.appendChild(assessmentSection->element());
    }
    
    return testPartElement;
}
