/***************************************************************************
 *   Copyright (C) 2010 by Volker Lanz <vl@fidra.de>                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 ***************************************************************************/

#if !defined(EDITPROFILEEQUIPMENTPAGE__H)

#define EDITPROFILEEQUIPMENTPAGE__H

#include <QWidget>

#include "ui_editprofileequipmentpage.h"

#include "core/equipment.h"

class Profile;

class QTreeWidgetItem;

class EditProfileEquipmentPage : public QWidget, public Ui::EditProfileEquipmentPage
{
	Q_OBJECT

	public:
		EditProfileEquipmentPage(QWidget* parent = NULL);
		~EditProfileEquipmentPage();

	signals:
		void modified();

	public:
		void setupPage(Profile* profile);
		void setupConnections();
		bool accept();
		void reject();

	protected:
		Equipment* selectedEquipment() const;
		QTreeWidgetItem* createEquipmentTreeNode(Equipment& e);
		void handleDeletedEquipment();

	protected slots:
		void onAddEquipmentButton(bool);
		void onRemoveEquipmentButton(bool);
		void onEquipmentItemChanged(QTreeWidgetItem* item, int col);
		void onEquipmentSelectionChanged();
		void onEquipmentPhotoButton(bool);

		void onEquipmentPurchaseDateChanged(const QDate&);
		void onEquipmentStatusChanged(int);
		void onEquipmentModelEditFinished();
		void onEquipmentBrandEditFinished();
		void onEquipmentTypeEditFinished();
		void onEquipmentWeightChanged(double);
		void onEquipmentNotesTextChanged();

	private:
		Profile* m_Profile;
		QSet<Equipment*> m_DeletedEquipments;
		Equipments m_BackupEquipments;
};

#endif

